/****************************************************************************
*                   particles.h
*
*  This module contains all defines, typedefs, and prototypes for particle.c.
*
*  from Persistence of Vision(tm) Ray Tracer
*  Copyright 1996,1999 Persistence of Vision Team
*---------------------------------------------------------------------------
Note: definition of ParticlePatch allows use of these features
*/
#ifndef PARTICLES_H
#define PARTICLES_H

/*****************************************************************************
* Global preprocessor defines
******************************************************************************/
#define SetBitFlag(var,flag,Bool)  { if(Bool){ (var) |=  (flag); } else { (var) &= ~(flag); }}

//opts flags
#define PARTICLES_ON	0x001
#define INCSTART_ON		0x002
#define INCEND_ON		0x004

//struct flags
#define COL_ON			0x0001
#define REFRAC_ON		0x0002
//#define SHAD_ON			0x0004
//#define CALC_ILLUM_ON	0x0008
//#define MULT_ILLUM_ON	0x0010
#define CHECK_RAD_ON	0x0020
#define INVERT_REFRAC	0x0040
//temporary for current ray
#define GET_CHECKED		0x0080

/*****************************************************************************
* Global typedefs
******************************************************************************/

typedef struct Particle_Struct PARTICLE;

struct Particle_Struct
{
	unsigned short flags;
	VECTOR center;
	COLOUR colour;
	DBL density;
	DBL size;
	DBL falloff;
	COLOUR illumcol;
	UV_VECT angle;
//	DBL nobenddist;
	DBL checkrad;
	DBL noise;
	DBL noisescale;

//Temporary stuff for each ray
	VECTOR point;//closest point on ray
	DBL pdist;//dist to that point
	DBL dist;//dist from the ray origin to that point

	PARTICLE* Next;
//	char* Label;
};


/*****************************************************************************
* Global functions
******************************************************************************/

PARTICLE* Parse_Particle(void);
PARTICLE* Parse_Particles(void);
void Parse_Particle_Stuff(PARTICLE*);
PARTICLE* Create_Particle (void);
void Copy_Particle (PARTICLE*,PARTICLE*);
PARTICLE* Duplicate_Particle (PARTICLE*);
void Destroy_Particle (PARTICLE*);
char BendRay(RAY *r, INTERSECTION* Intersection);
void do_particle_colours(RAY *Ray, INTERSECTION *Intersection, COLOUR Colour);

/*****************************************************************************
* Global variables
******************************************************************************/
extern PARTICLE* Bent_Particle;

#endif// PARTICLES_H
